package Bundle::Text::MagicTemplate;
$VERSION = '1.00';
__END__

=head1 NAME

Bundle::Text::MagicTemplate - A bundle to install Text::MagicTemplate plus all related extensions and prerequisites.

=head1 SYNOPSIS

    perl -MCPAN -e 'install Bundle::Text::MagicTemplate'

=head1 CONTENTS

Text::MagicTemplate         - the main distribution

HTML::Tagset                - used by HTML::Parser

HTML::Parser                - used by HTML::FillInForm and HTML::TableTiler

HTML::TableTiler            - used by Text::MagicTemplateX::HTML

HTML::FillInForm            - used by Text::MagicTemplateX::HTML

Text::MagicTemplateX::HTML  - the HTML extensions collection

=head1 DESCRIPTION

This bundle gathers together Text::MagicTemplate and all the related extensions and prerequisites.

Note: A Bundle is a module that simply defines a collection of other modules. It is used by the CPAN module to automate the fetching, building and installing of modules from the CPAN ftp archive sites.

=head1 AUTHOR

Domizio Demichelis, <dd@4pro.net>.

=head1 COPYRIGHT

Copyright (c)2002 Domizio Demichelis. All Rights Reserved. This is free software; it may be used freely and redistributed for free providing this copyright header remains part of the software. You may not charge for the redistribution of this software. Selling this code without Domizio Demichelis' written permission is expressly forbidden.

This software may not be modified without first notifying the author (this is to enable me to track modifications). In all cases the copyright header should remain fully intact in all modifications.

This code is provided on an "As Is'' basis, without warranty, expressed or implied. The author disclaims all warranties with regard to this software, including all implied warranties of merchantability and fitness, in no event shall the author, be liable for any special, indirect or consequential damages or any damages whatsoever including but not limited to loss of use, data or profits. By using this software you agree to indemnify the author from any liability that might arise from it is use. Should this code prove defective, you assume the cost of any and all necessary repairs, servicing, correction and any other costs arising directly or indrectly from it is use.

The copyright notice must remain fully intact at all times. Use of this software or its output, constitutes acceptance of these terms.